@@ -1,3 +1,5 @@
+# 2018-12-06 CJS created initial plot
+# 2018-11-27 CJS removed openBugs
 # 2014-09-01 CJS conversion to JAGS
 #                - no model name
 #                - C(,20) -> T(,20)
@@ -17,6 +19,9 @@
 # 2009-12-05 CJS added title to argument list
 # 2009-12-01 CJS added openbugs/winbugs directory to argument list
 
+#' @keywords internal 
+
+
 TimeStratPetersenDiagErrorWHSteel <-
   function(title, prefix,
            time, n1, m2,
@@ -29,8 +34,8 @@ TimeStratPetersenDiagErrorWHSteel <-
            tau_xiP=1/var( logit((m2+.5)/(n1+1)),na.rm=TRUE),
            tauP.alpha=.001, tauP.beta=.001,
            debug=FALSE, debug2=FALSE,
-	   engine=c('jags','openbugs')[1], 
-           InitialSeed){
+           InitialSeed,
+           save.output.to.files=TRUE){
 
 set.seed(InitialSeed)  # set prior to initial value computations
 
@@ -137,70 +142,19 @@ model {
    ##### Fit the spline for W.YoY - this covers the entire experiment ######
    for(i in 1:Nstrata){
         logUne.W.YoY[i] <- inprod(SplineDesign.W.YoY[i,1:n.bU.W.YoY],bU.W.YoY[1:n.bU.W.YoY])  # spline design matrix * spline coeff
-", fill=TRUE)
-sink()  # Temporary end of saving bugs program
-if(tolower(engine)=="jags") {
-   sink("model.txt", append=TRUE)
-   cat("
         etaU.W.YoY[i] ~ dnorm(logUne.W.YoY[i], taueU)T(,20)          # add random error
-   ",fill=TRUE)
-   sink()
-}
-if(tolower(engine) %in% c("openbugs")) {
-   sink("model.txt", append=TRUE)
-   cat("
-        etaU.W.YoY[i] ~ dnorm(logUne.W.YoY[i], taueU)C(,20)          # add random error
-   ",fill=TRUE)
-   sink()
-}
-   sink("model.txt", append=TRUE)
-   cat("
         eU.W.YoY[i] <- etaU.W.YoY[i] - logUne.W.YoY[i]
    }
    ##### Fit the spline for W.1 -   this covers the entire experiment ######
    for(i in 1:Nstrata){
         logUne.W.1[i] <- inprod(SplineDesign.W.1[i,1:n.bU.W.1],bU.W.1[1:n.bU.W.1])  # spline design matrix * spline coeff
-   ", fill=TRUE)
-sink()  # Temporary end of saving bugs program
-if(tolower(engine)=="jags") {
-   sink("model.txt", append=TRUE)
-   cat("
         etaU.W.1[i] ~ dnorm(logUne.W.1[i], taueU)T(,20)              # add random error
-   ",fill=TRUE)
-   sink()
-}
-if(tolower(engine) %in% c("openbugs")) {
-   sink("model.txt", append=TRUE)
-   cat("
-        etaU.W.1[i] ~ dnorm(logUne.W.1[i], taueU)C(,20)              # add random error
-   ",fill=TRUE)
-   sink()
-}
-   sink("model.txt", append=TRUE)
-   cat("
         eU.W.1[i] <- etaU.W.1[i] - logUne.W.1[i]
    }
    ##### Fit the spline for hatchery fish - these fish only enter AFTER hatch.after ######
    for(i in (hatch.after+1):Nstrata){
         logUne.H.1[i] <- inprod(SplineDesign.H.1[i,1:n.bU.H.1],bU.H.1[1:n.bU.H.1])  # spline design matrix * spline coeff
-   ", fill=TRUE)
-sink()  # Temporary end of saving bugs program
-if(tolower(engine)=="jags") {
-   sink("model.txt", append=TRUE)
-   cat("
         etaU.H.1[i] ~ dnorm(logUne.H.1[i], taueU)T(,20)              # add random error
-   ",fill=TRUE)
-   sink()
-}
-if(tolower(engine) %in% c("openbugs")) {
-   sink("model.txt", append=TRUE)
-   cat("
-        etaU.H.1[i] ~ dnorm(logUne.H.1[i], taueU)C(,20)              # add random error
-   ",fill=TRUE)
-   sink()
-}
-   sink("model.txt", append=TRUE)
-   cat("
         eU.H.1[i] <- etaU.H.1[i] - logUne.H.1[i]
    }
 
@@ -233,11 +187,6 @@ if(tolower(engine) %in% c("openbugs")) {
 
    ##### Hyperpriors #####
    ## Run size - wild and hatchery fish - flat priors
-      ", fill=TRUE)
-sink()  # Temporary end of saving bugs program
-if(tolower(engine)=="jags") {
-   sink("model.txt", append=TRUE)
-   cat("
    for(i in 1:n.b.flat.W.YoY){
       bU.W.YoY[b.flat.W.YoY[i]] ~ dnorm(0, 1E-6)
    }
@@ -247,26 +196,7 @@ if(tolower(engine)=="jags") {
    for(i in 1:n.b.flat.H.1){
       bU.H.1[b.flat.H.1[i]] ~ dnorm(0, 1E-6)
    }
-   ",fill=TRUE)
-   sink()
-}
-if(tolower(engine) %in% c("openbugs")) {
-   sink("model.txt", append=TRUE)
-   cat("
-   for(i in 1:n.b.flat.W.YoY){
-      bU.W.YoY[b.flat.W.YoY[i]] ~ dflat()
-   }
-   for(i in 1:n.b.flat.W.1){
-      bU.W.1[b.flat.W.1[i]] ~ dflat()
-   }
-   for(i in 1:n.b.flat.H.1){
-      bU.H.1[b.flat.H.1[i]] ~ dflat()
-   }
-   ",fill=TRUE)
-   sink()
-}
-   sink("model.txt", append=TRUE)
-   cat("
+
    ## Run size - priors on the difference for wild and hatchery fish
    for(i in 1:n.b.notflat.W.YoY){
       xiU.W.YoY[b.notflat.W.YoY[i]] <- 2*bU.W.YoY[b.notflat.W.YoY[i]-1] - bU.W.YoY[b.notflat.W.YoY[i]-2]
@@ -440,6 +370,26 @@ SplineDesign.H.1 <- rbind(matrix(0,nrow=hatch.after, ncol=ncol(SplineDesign.H.1)
 
 
 #browser()
+# Initial plot
+# create an initial plot of the fit to the number of YoY and Age1 unmarked fish
+  plot.data <- rbind(data.frame(time=time, group="H.1", pch="H",
+                                logUguess = log(Uguess.H.1+1),
+                                spline=SplineDesign.H.1 %*% init.bU.H.1, stringsAsFactors=FALSE),
+                     data.frame(time=time, group="W.1", pch="W",
+                                logUguess = log(Uguess.W.1+1),
+                                spline=SplineDesign.W.1 %*% init.bU.W.1, stringsAsFactors=FALSE),
+                     data.frame(time=time, group="W.YoY", pch="w",
+                                logUguess = log(Uguess.W.YoY+1),
+                                spline=SplineDesign.W.YoY %*% init.bU.W.YoY, stringsAsFactors=FALSE))
+  init.plot <- ggplot(data=plot.data, aes_(x=~time, color=~group, shape=~group))+
+     ggtitle(title, subtitle="Initial spline fit to estimated log U[i] for W and H")+
+     geom_point(aes_(y=~logUguess), position=position_dodge(width=0.2))+
+     geom_line(aes_(y=~spline),     position=position_dodge(width=0.2))+
+     xlab("Stratum")+ylab("log(U[i])")+
+     theme(legend.position=c(0,0), legend.justification=c(0,0))
+  if(save.output.to.files)ggsave(init.plot, filename=paste(prefix,"-initialU.pdf",sep=""), height=4, width=6, units="in")
+  #results$plots$plot.init <- init.plot  # do this after running the MCMC chain (see end of function)
+
 
 # get the logitP=logit(P) covariate matrix ready
 logitP.cov <- as.matrix(logitP.cov)
@@ -533,8 +483,9 @@ results <- run.MCMC(modelFile=model.file,
                         overRelax=FALSE,
                         initialSeed=InitialSeed,
                         working.directory=working.directory,
-			engine=engine,
                         debug=debug)
 
+results$plots$plot.init <- init.plot  # do this after running the MCMC chain (see end of function)
+
 return(results)
 }
