@@ -1,3 +1,14 @@
+## 2018-12-18 CJS deprecation of sampling fraction 
+## 2018-12-06 CJS converted report to textConnection
+## 2018-12-03 CJS converted fit plot to ggplot
+## 2018-12-02 CJS converted traceplots to ggplot
+## 2018-12-01 CJS created posterior plots using ggplot
+## 2018-11-30 CJS created acf plot using ggplot
+## 2018-11-30 CJS Fixed problem of epsilon not being right length
+## 2018-11-28 CJS Fixed problem of printing results getting cutoff if too large
+## 2018-11-27 CJS Added explicit library refrences
+## 2018-11-25 CJS Removed all references to OpenBugs
+## 2015-06-10 CJS gof converted to ggplot()
 ## 2014-09-01 CJS conversion to jags
 ## 2012-08-30 CJS fixed errors with any() and all() with NAs in error checking
 ## 2012-02-15 CJS fixed plotting limits on log(U) plot
@@ -13,8 +24,143 @@
 ## 2010-03-12 CJS added n.chains etc to argument list; added optional call for debugging purposes
 ## 2010-03-03 SJB Created File
 
+
+#' Wrapper (*_fit)  to fit the Time Stratified Petersen Estimator
+#' with NON Diagonal Entries function and a non-parametric travel time estimator..
+#' 
+#' Takes the number of marked fish released, the number of recaptures, and the
+#' number of unmarked fish and uses Bayesian methods to fit a fit a spline
+#' through the population numbers and a hierarchical model for the trap
+#' efficiencies over time.  The output is written to files and an MCMC object
+#' is also created with samples from the posterior.
+#' 
+#' Normally the user makes a call to the *_fit function which then calls the
+#' fitting function.
+#' 
+#' Use the \code{\link{TimeStratPetersenDiagError_fit}} function for cases
+#' where recaptures take place ONLY in the stratum of release, i.e. the
+#' diagonal case.
+#' 
+#' The *NP functions fit a non-parametric distribution for the travel times.
+#' 
+#' @param title A character string used for a title on reports and graphs
+#' @param prefix A character string used as the prefix for created files. All
+#' created graph files are of the form prefix-xxxxx.pdf.
+#' @param time A numeric vector of time used to label the strata. For example,
+#' this could be julian week for data stratified at a weekly level.
+#' @param n1 A numeric vector of the number of marked fish released in each
+#' time stratum.
+#' @param m2 A numeric matrix of the number of fish released in stratum [i] and
+#' recovered in [j-1] strata later.  For example m2[3,5] is the number of
+#' marked fish released in stratum 3 and recovered 4 strata later in stratum 7.
+#' The first column is the number of marked fish recovered in the stratum of
+#' release, i.e. 0 strata later.  Use the
+#' \code{\link{TimeStratPetersenDiagError_fit}} function for cases where
+#' recaptures take place ONLY in the stratum of release, i.e. the diagonal
+#' case.
+#' @param u2 A numeric vector of the number of unmarked fish captured in each
+#' stratum.  These will be expanded by the capture efficiency to estimate the
+#' population size in each stratum. The length of u2 should be between the length of n1 and length n1 + number of columns in m2 -1
+#' @param sampfrac \strong{Deprecated.} DO NOT USE ANYMORE. A numeric vector with entries between 0 and 1 indicating
+#' what fraction of the stratum was sampled. For example, if strata are
+#' calendar weeks, and sampling occurred only on 3 of the 7 days, then the
+#' value of \code{sampfrac} for that stratum would be 3/7.
+#' @param jump.after A numeric vector with elements belonging to \code{time}.
+#' In some cases, the spline fitting the population numbers should be allowed
+#' to jump.  For example, the population size could take a jump when hatchery
+#' released fish suddenly arrive at the trap.  The jumps occur AFTER the strata
+#' listed in this argument.
+#' @param bad.n1 A numeric vector with elements belonging to \code{time}.  In
+#' some cases, something goes wrong in the stratum, and the number of marked
+#' fish releases should be discarded.  The values of \code{n1} will be set to
+#' NA for these strata.
+#' @param bad.m2 A numeric vector with elements belonging to \code{time}.  In
+#' some cases, something goes wrong in the stratum, and the number of recovered
+#' marked fish should be ignored. For example, poor handling is suspected to
+#' induce handling induced mortality in the marked fish and so only very few
+#' are recovered.  The values of \code{m2} in the entire row will be set to NA
+#' for these strata.
+#' @param bad.u2 A numeric vector with elements belonging to \code{time}.  In
+#' some cases, something goes wrong in the stratum, and the number of unmarked
+#' fish captured should be ignored.  The values of \code{u2} in the entire row
+#' will be set to NA for these strata.
+#' @param logitP.cov A numeric matrix for covariates to fit the
+#' logit(catchability).  Default is a single intercept, i.e. all strata have
+#' the same mean logit(catchability).
+#' @param logitP.fixed A numeric vector (could be null) of the time strata
+#' where the logit(P) would be fixed. Typically, this is used when the capture
+#' rates for some strata are 0 and logit(P) is set to -10 for these strata. The
+#' fixed values are given in \code{logitP.fixed.values}
+#' @param logitP.fixed.values A numerical vector (could be null) of the fixed
+#' values for logit(P) at strata given by logitP.fixed. Typically this is used
+#' when certain strata have a 0 capture rate and the fixed value is set to -10
+#' which on the logit scale gives p[i] essentially 0. Don't specify values such
+#' as -50 because numerical problems could occur in WinBugs/OpenBugs.
+#' @param n.chains Number of parallel MCMC chains to fit.
+#' @param n.iter Total number of MCMC iterations in each chain.
+#' @param n.burnin Number of burn-in iterations.
+#' @param n.sims Number of simulated values to keeps for posterior
+#' distribution.
+#' @param tauU.alpha One of the parameters along with \code{tauU.beta} for the
+#' prior for the variance of the random noise for the smoothing spline.
+#' @param tauU.beta One of the parameters along with \code{tauU.alpha} for the
+#' prior for the variance of the random noise for the smoothing spline.
+#' @param taueU.alpha One of the parameters along with \code{taueU.beta} for
+#' the prior for the variance of noise around the spline.
+#' @param taueU.beta One of the parameters along with \code{taueU.alpha} for
+#' the prior for the variance of noise around the spline.
+#' @param Delta.max Maximum transition time for marked fish, i.e. all fish
+#' assumed to have moved by Delta.max unit of time
+#' @param tauTT.alpha One of the parameters along with \code{tauTT.beta} for
+#' the prior on 1/var of logit continuation ratio for travel times
+#' @param tauTT.beta One of the parameters along with \code{tauTT.alpha} for
+#' the prior on 1/var of logit continuation ratio for travel times
+#' @param mu_xiP One of the parameters for the prior for the mean of the
+#' logit(catchability) across strata
+#' @param prior.muTT - prior for movement rates.
+#'                    These are like a dirchelet type prior
+#'                    where x are values representing belief in the travel times.
+#'                    For example, x=c(1,4,3,2) represents a system where the
+#'                   maximum travel time is 3 strata after release with
+#'                    1/10=.1 of the animals moving in the stratum of release
+#'                    4/10=.4 of the animals taking 1 stratum to move etc
+#'                    So if x=c(10,40,30,20), this represent the same movement pattern
+#'                    but a strong degree of belief
+
+#' @param tau_xiP One of the parameter for the prior for the mean of the
+#' logit(catchability) across strata
+#' @param tauP.alpha One of the parameters for the prior for the variance in
+#' logit(catchability) among strata
+#' @param tauP.beta One of the parameters for the prior for the variance in
+#' logit(catchability) among strata
+#' @param run.prob Numeric vector indicating percentiles of run timing should
+#' be computed.
+#' @param debug Logical flag indicating if a debugging run should be made. In
+#' the debugging run, the number of samples in the posterior is reduced
+#' considerably for a quick turn around.
+#' @param debug2 Logical flag indicated if additional debugging information is
+#' produced. Normally the functions will halt at \code{browser()} calls to
+#' allow the user to peek into the internal variables. Not useful except to
+#' package developers.
+#' @param InitialSeed Numeric value used to initialize the random numbers used
+#' in the MCMC iterations.
+#' @param save.output.to.files Should the plots and text output be save to the files
+#' in addition to being stored in the MCMC object? 
+#' 
+#' @return An MCMC object with samples from the posterior distribution. A
+#' series of graphs and text file are also created in the working directory.
+#' @template author 
+#' @template references
+#' @keywords ~models ~smooth
+#' @examples
+#'  
+#' ##---- See the vignette  for examples of how to use this package
+#' ##
+#' 
+#' @export TimeStratPetersenNonDiagErrorNP_fit
+
 TimeStratPetersenNonDiagErrorNP_fit<- function( title="TSPNDENP", prefix="TSPNDENP-",
-                         time, n1, m2, u2, sampfrac, jump.after=NULL,
+                         time, n1, m2, u2, sampfrac=rep(1,length(u2)), jump.after=NULL,
                          bad.n1=c(), bad.m2=c(), bad.u2=c(),
                          logitP.cov=rep(1,length(u2)),
                          logitP.fixed=NULL, logitP.fixed.values=NULL,
@@ -28,14 +174,14 @@ TimeStratPetersenNonDiagErrorNP_fit<- function( title="TSPNDENP", prefix="TSPNDE
                          tauTT.alpha=.1,tauTT.beta=.1,
                          run.prob=seq(0,1,.1),  # what percentiles of run timing are wanted
                          debug=FALSE, debug2=FALSE,
-                         engine=c('jags','openbugs')[1],
-                         InitialSeed=ceiling(runif(1,min=0, max=if(engine=="jags"){1000000}else{14}))) {
+                         InitialSeed=ceiling(runif(1,min=0,1000000)),
+                         save.output.to.files=TRUE) {
   ## Fit a Time Stratified Petersen model with NON-diagonal entries and with smoothing on U allowing for random error
   ## This is the classical stratified Petersen model where the recoveries can take place for this and multiple
   ## strata later. Transisions of marked fish are modelled non-parametrically.
   ##
-
-  version <- '2014-09-01'
+  
+  version <- '2020-01-01'
   options(width=200)
 
   ## Input parameters are
@@ -54,7 +200,7 @@ TimeStratPetersenNonDiagErrorNP_fit<- function( title="TSPNDENP", prefix="TSPNDE
   ##             The vector u2 should be long enough to account for any fish that are recaptured later on
   ##             from releases late in the season. The bottom right diagonal of m2 may be all zeros - that is ok
   ##             Notice that length(u2) can be longer than length(n1)+nrow(m2).
-  ##    sampfrac - sampling fraction to adjust for how many days of the week was the trap operating
+  ##    sampfrac - Deprecated. Do not use anymore. sampling fraction to adjust for how many days of the week was the trap operating
   ##              This is expressed as fraction i.e. 3 days out of 7 is expressed as 3/7=.42 etc.
   ##              If the trap was operating ALL days, then the SampFrac = 1. It is possible for the sampling
   ##              fraction to be > 1 (e.g. a mark is used for 8 days instead of 7. The data are adjusted
@@ -74,7 +220,7 @@ TimeStratPetersenNonDiagErrorNP_fit<- function( title="TSPNDENP", prefix="TSPNDE
   ##               CAUTION - this MAY not be what you want to do. It is likely best to enter ALL strata
   ##               if you have any covariates. The default, if not specified, is a constant (the mean logit)
   ##    prior.muTT - prior for movement rates.
-  ##                  These are like a dirchelete type prior
+  ##                  These are like a dirchelet type prior
   ##                  where x are values representing belief in the travel times.
   ##                  For example, x=c(1,4,3,2) represents a system where the
   ##                  maximum travel time is 3 strata after release with
@@ -113,9 +259,9 @@ sampfrac <- as.vector(sampfrac)
          length(u2)," ",length(sampfrac)," ",length(time),"\n")
     return()}
 
-    if(length(logitP.cov) %% length(u2) != 0){
+  if(length(logitP.cov) %% length(u2) != 0){
       cat("***** ERROR ***** Dimension of covariate vector doesn't match length of u2. They are:",
-        length(u2)," ",length(logitP.cov)," ",dim(logitP.cov),"\n")
+         length(u2)," ",length(logitP.cov)," ",dim(logitP.cov),"\n")
       return()}
 
   ##  2. Check that rowsum of m2<= n1
@@ -127,29 +273,31 @@ sampfrac <- as.vector(sampfrac)
 
   ##  3. Elements of bad.m2 and jump.after must belong to time
   if(!all(bad.n1 %in% time,na.rm=TRUE)){
-    cat("***** ERROR ***** bad.n1 must be elements of strata identifiers. You entered \n bad.n1:",bad.n1,
+    cat("***** ERROR ***** bad.n1 must be elements of strata identifiers. You entered \n bad.n1:",
+        paste(bad.n1,collapse=","),
          "\n Strata identifiers are \n time:",paste(time,collapse=","), "\n")
     return()}
 
   if(!all(bad.m2 %in% time,na.rm=TRUE)){
-    cat("***** ERROR ***** bad.m2 must be elements of strata identifiers. You entered \n bad.m2:",bad.m2,
+    cat("***** ERROR ***** bad.m2 must be elements of strata identifiers. You entered \n bad.m2:",
+        paste(bad.m2,collapse=","),
          "\n Strata identifiers are \n time:",paste(time,collapse=","), "\n")
     return()}
 
   if(!all(bad.u2 %in% time,na.rm=TRUE)){
-    cat("***** ERROR ***** bad.u2 must be elements of strata identifiers. You entered \n bad.u2:",bad.u2,
+    cat("***** ERROR ***** bad.u2 must be elements of strata identifiers. You entered \n bad.u2:",paste(bad.u2,collapse=","),
          "\n Strata identifiers are \n time:",paste(time,collapse=","), "\n")
     return()}
 
   if(!all(jump.after %in% time,na.rm=TRUE)){
-    cat("***** ERROR ***** jump.after must be elements of strata identifiers. You entered \n jump.after:",jump.after,
+    cat("***** ERROR ***** jump.after must be elements of strata identifiers. You entered \n jump.after:",paste(jump.after,collapse=","),
          "\n Strata identifiers are \n time:",paste(time,collapse=","), "\n")
     return()}
 
 
   #  5. check that index of logitP.fixed belong to time
   if(!all(logitP.fixed %in% time,na.rm=TRUE)){
-    cat("***** ERROR ***** logitP.fixed must be elements of strata identifiers. You entered \n logitP.fixed:",logitP.fixed,
+    cat("***** ERROR ***** logitP.fixed must be elements of strata identifiers. You entered \n logitP.fixed:",paste(logitP.fixed,collapse=","),
         "\n Strata identifiers are \n time:",paste(time,collapse=","), "\n")
     return()}
   if(length(logitP.fixed)!=length(logitP.fixed.values)){
@@ -162,6 +310,17 @@ sampfrac <- as.vector(sampfrac)
     cat("***** ERROR ***** Prior muTT must have same length as columns of m2. You entered \n prior.muTT:",prior.muTT,"\n")
     return()}
 
+  #7 check that the length of u2 
+  if(length(u2) < length(n1) | length(u2) > (length(n1)+ ncol(m2)-1)){
+    cat("***** ERROR ***** Length(u2) must between length(n1) and length(n1)+ncol(m2) \n")
+    return()
+  }
+
+  # Deprecation of sampling fraction.
+  if(any(sampfrac != 1)){
+    cat("***** ERROR ***** Sampling fraction is deprecated for any values other than 1. DO NOT USE ANYMORE. ")
+    return()
+  }
 
 
   ## Define maximum travel time if not supplied by user
@@ -182,16 +341,16 @@ sampfrac <- as.vector(sampfrac)
   results.filename <- paste(prefix,"-results.txt",sep="")
 
   ## Open sink to output file
-  sink(results.filename)
+  stdout <- vector('character')
+  report <- textConnection('stdout', 'wr', local = TRUE)
+  sink(report)
+  
+  
   cat(paste("Time Stratified Petersen with Non-Diagonal recaptures, error in smoothed U, and non-parametric modelling of travel times - ", date()))
   cat("\nVersion: ", version)
 
   cat("\n\n", title, "Results \n\n")
 
-  ## length(sampfrac) =length(u2)
-  ## m2(i,+) < n1(i)
-  ## 0 < sampfrac < 1
-
   cat("*** Raw data *** (padded to match length of u2) \n")
   jump.indicator <- rep('   ', length(u2))
   jump.indicator[time %in% jump.after]<- '***'
@@ -213,76 +372,28 @@ sampfrac <- as.vector(sampfrac)
   if(length(logitP.fixed)==0) cat("none - NO fixed values")
 
   ## Obtain the Pooled Petersen estimator prior to fixup of bad.n1, bad.m2, and bad.u2 values
-  cat("\n\n*** Pooled Petersen Estimate prior to fixing bad n1, m2, or u2 values  CHECK - CHECK - CHECK - CHECK ***\n\n")
+  cat("\n\n*** Pooled Petersen Estimate prior to fixing bad n1, m2, or u2 values\n\n")
   temp.n1 <- n1
   temp.m2 <- m2
   temp.u2 <- u2
 
   cat("Total n1=", sum(temp.n1,na.rm=TRUE),";  m2=",sum(temp.m2,na.rm=TRUE),";  u2=",sum(temp.u2,na.rm=TRUE),"\n\n")
   pp <- SimplePetersen(sum(temp.n1,na.rm=TRUE), sum(temp.m2,na.rm=TRUE), sum(temp.u2,na.rm=TRUE))
-  cat("Est U(total) ", format(round(pp$est),big.mark=","),"  (SE ", format(round(pp$se), big.mark=","), ")\n\n\n")
+  cat("Est U(total) ", format(round(pp$U.est),big.mark=","),"  (SE ", format(round(pp$U.se), big.mark=","), ")\n")
+  cat("Est N(total) ", format(round(pp$N.est),big.mark=","),"  (SE ", format(round(pp$N.se), big.mark=","), ")\n\n\n")
+  
 
-  ## Obtain the Pooled Petersen estimator after removal of entries with bad.n1, m2, or u2 values
-  ## select <- !(time %in% bad.n1 | time %in% bad.m2 | time %in% bad.u2)
-  select <- (temp.n1>0) & (!is.na(n1)) & (!apply(is.na(temp.m2),1,any)) & (!is.na(temp.u2[1:length(n1)]))
-  cat("\n\n*** Pooled Petersen Estimate after fixing bad m2 values  CHECK - CHECK - CHECK - CHECK ***\n\n")
-  cat("The following strata were excluded:",
-      if(length(time[!select])>0){time[!select]} else {" NONE"}, "\n")
-
-  ##temp.n1 <- n1[select]
-  ##temp.m2 <- m2[select]
-  ##temp.u2 <- u2[select]
-
-  cat("Total n1=", sum(temp.n1,na.rm=TRUE),";  m2=",sum(temp.m2,na.rm=TRUE),";  u2=",sum(temp.u2, na.rm=TRUE),"\n\n")
-  pp <- SimplePetersen(sum(temp.n1,na.rm=TRUE), sum(temp.m2,na.rm=TRUE), sum(temp.u2,na.rm=TRUE))
-  cat("Est U(total) ", format(round(pp$est),big.mark=","),"  (SE ", format(round(pp$se), big.mark=","), ")\n\n\n")
-
-
-################ This needs more thoughs ##################
-  ## Obtain Petersen estimator for each stratum prior to removing bad m2 values
-  ##cat("*** Stratified Petersen Estimator for each stratum PRIOR to removing bad m2 values ***\n\n")
-  ##temp.n1 <- n1
-  ##temp.m2 <- m2
-  ##temp.u2 <- u2
-  ##sp <- SimplePetersen(temp.n1, temp.m2, temp.u2)
-  ##temp <- cbind(time, temp.n1, temp.m2, temp.u2, round(sp$est), round(sp$se))
-  ##colnames(temp) <- c('time', 'n1','m2','u2', 'U[i]', 'SE(U[i])')
-  ##print(temp)
-  ##cat("\n")
-  ##cat("Est U(total) ", format(round(sum(sp$est, na.rm=TRUE)),big.mark=","),
-  ##   "  (SE ", format(round(sqrt(sum(sp$se^2, na.rm=TRUE))), big.mark=","), ")\n\n\n")
-
-
-  ## Obtain Petersen estimator for each stratum after removing bad m2 values
-  ##cat("*** Stratified Petersen Estimator for each stratum AFTER removing bad m2 values ***\n\n")
-  ##temp.n1 <- n1
-  ##temp.m2 <- m2
-  ##temp.m2[index.bad.m2] <- NA
-  ##temp.u2 <- u2
-  ##sp <- SimplePetersen(temp.n1, temp.m2, temp.u2)
-  ##temp <- cbind(time, temp.n1, temp.m2, temp.u2, round(sp$est), round(sp$se))
-  ##colnames(temp) <- c('time', 'n1','m2','u2', 'U[i]', 'SE(U[i])')
-  ##print(temp)
-  ##cat("\n")
-  ##cat("Est U(total) ", format(round(sum(sp$est)),big.mark=","),
-  ##    "  (SE ", format(round(sqrt(sum(sp$se^2))), big.mark=","), ")\n\n\n")
-
-
-
-
-############## This needs more thought ##########################
   ## Test if pooling can be done
-  ##cat("*** Test if pooled Petersen is allowable. [Check if marked fractions are equal] ***\n\n")
-  ##select <- (n1>0) & (!is.na(n1)) & (!is.na(temp.m2))
-  ##temp.n1 <- n1[select]
-  ##temp.m2 <- m2[select]
-  ##test <- TestIfPool( temp.n1, temp.m2)
-  ##cat("(Large Sample) Chi-square test statistic ", test$chi$statistic," has p-value", test$chi$p.value,"\n\n")
-  ##temp <- cbind(time[select],test$chi$observed, round(test$chi$expected,1), round(test$chi$residuals^2,1))
-  ##colnames(temp) <- c('time','n1-m2','m2','E[n1-m2]','E[m2]','X2[n1-m2]','X2[m2]')
-  ##print(temp)
-  ##cat("\n Be cautious of using this test in cases of small expected values. \n\n")
-
+  cat("*** Test if pooled Petersen is allowable. [Check if fraction captured equal] ***\n\n")
+  select <- (n1>0) & (!is.na(n1)) & (!is.na(apply(m2,1,sum)))
+  temp.n1 <- n1[select]
+  temp.m2 <- m2[select,]
+  test <- TestIfPool( temp.n1, apply(temp.m2,1,sum))
+  cat("(Large Sample) Chi-square test statistic ", test$chi$statistic," has p-value", test$chi$p.value,"\n\n")
+  temp <- cbind(time[1:length(n1)][select],test$chi$observed, round(test$chi$expected,1), round(test$chi$residuals^2,1))
+  colnames(temp) <- c('time','n1-m2*','m2*','E[n1-m2]','E[m2]','X2[n1-m2]','X2[m2]')
+  print(temp)
+  cat("\n Be cautious of using this test in cases of small expected values. \n\n")
 
   ## Adjust the data for the explicity bad values or other problems
   new.time <- time
@@ -292,25 +403,8 @@ sampfrac <- as.vector(sampfrac)
   new.sampfrac <- sampfrac
   new.logitP.cov <- logitP.cov
 
-  ## If n1=m2=0, then set n1 to 1, and set m2<-NA as winbugs cannot deal with n1=0 and m2=0
-  new.m2[new.n1==0,] <- NA
-  new.n1[new.n1==0 ] <- 1
-
-
-
-#################### This needs more thought ####################
-  ## Adjust data when a stratum has less than 100% sampling fraction to "estimate" the number
-  ## of unmarked fish that were captured. It is not necessary to adjust the n1 and m2 values
-  ## as these are used ONLY to estimate the capture efficiency.
-  ## In reality, there should be a slight adjustment
-  ## to the precision to account for this change, but this is not done.
-  ## Similarly, if the sampling fraction is more than 1, the adjustment is made back to a standard week.
-  ##new.u2 <- round(new.u2/new.sampfrac)
-
-  ## Adjust for strata where sampling fraction=0. On these strata
-  ## u2 is set to NA so that there is NO information on U2 for this stratum
-  new.u2[new.sampfrac<.001 & new.sampfrac >0] <- NA
-
+  
+ 
   ## Set the bad values to missing
   new.n1[time[1:length(n1)] %in% bad.n1]  <- NA
   new.m2[time[1:length(n1)] %in% bad.m2,] <- NA
@@ -328,21 +422,50 @@ sampfrac <- as.vector(sampfrac)
   cat("\n\n")
 
   ## The NP analysis does not need the expanded m2 array, but this is
-  ## needed late on. So, we'd better compute it here. The last column
+  ## needed later on. So, we'd better compute it here. The last column
   ## of this matrix will be the number of individuals from each
   ## stratum that are not recaptured.
   ##
 
-  expanded.m2 <- matrix(0, nrow=length(new.n1), ncol=length(new.u2)+1)
+  #browser()
+  expanded.m2 <- matrix(0, nrow=length(new.n1), ncol=length(new.n1)+ncol(m2)+1)
   for(i in 1:length(new.n1)){
-    expanded.m2[i,1:length(new.u2)] <- c(rep(0,i-1),new.m2[i,],rep(0,length(new.u2)))[1:length(new.u2)]
-    expanded.m2[i,length(new.u2)+1] <- new.n1[i] - sum(new.m2[i,])
+    expanded.m2[i,1:(ncol(expanded.m2)-1)] <- c(rep(0,i-1),new.m2[i,],rep(0,ncol(expanded.m2)))[1:(ncol(expanded.m2)-1)]
+    expanded.m2[i,ncol(expanded.m2)] <- new.n1[i] - sum(new.m2[i,])
+  }
+  
+  cat("*** Expanded m2 array with column sum and u2 ***\n\n")
+  save.max.print <- getOption("max.print")
+  options(max.print=.Machine$integer.max)
+ 
+  temp <- rbind(expanded.m2, apply(expanded.m2,2,sum, na.rm=TRUE))
+  rownames(temp)[nrow(temp)] <- 'Column totals'
+  temp <- rbind(temp, c(u2, rep(NA, ncol(expanded.m2)-length(u2)) ))
+  rownames(temp)[nrow(temp)] <- "Untagged (u2)"
+  temp <- rbind(temp, c(new.u2, rep(NA, ncol(expanded.m2)-length(new.u2)) ))
+  rownames(temp)[nrow(temp)] <- "Untagged - after fixups"
+  
+  new.logitP.fixed <- rep(NA, length(new.u2))
+  new.logitP.fixed[match(logitP.fixed, time)] <- logitP.fixed.values
+  
+  temp <- rbind(temp, c(new.logitP.fixed, rep(NA, ncol(expanded.m2)-length(new.u2)) ))
+  rownames(temp)[nrow(temp)] <- "Logit P fixed"
+  rownames(temp)[1:length(n1)] <- 1:length(n1)
+  print(temp)
+  options(max.print=save.max.print)
+  sink()
+  
+  # some further checking on u2. Make sure that every columns where there are recoveries has a u2
+  if( (length(u2)+1) <= (ncol(temp)-1)) {
+     if(any( temp["Column totals", (length(u2)+1):(ncol(temp)-1)] >0)){
+       cat("***** ERROR ***** Non-zero recoveries and u2 not available at end of experiment??? \n Check above matrix\n")
+       return()
+     }
   }
 
-  cat("*** Expanded m2 array ***\n\n")
-  print(expanded.m2)
+  sink(report, append=TRUE)
 
-  # assign the logitP fixed values etc.
+  # assign the logitP fixed values etc (replicates what was done above, but convenient to put here)
   new.logitP.fixed <- rep(NA, length(new.u2))
   new.logitP.fixed[match(logitP.fixed, time)] <- logitP.fixed.values
 
@@ -406,7 +529,7 @@ sampfrac <- as.vector(sampfrac)
                          mean.muTT=mean.muTT, sd.muTT=sd.muTT,
                          tauTT.alpha=tauTT.alpha,tauTT.beta=tauTT.beta,
                          debug=debug, debug2=debug2, 
-			 engine=engine, InitialSeed=InitialSeed)
+			                   InitialSeed=InitialSeed, save.output.to.files=save.output.to.files)
    } else #notice R syntax requires { before the else
    {results <- TimeStratPetersenNonDiagErrorNP(title=title, prefix=prefix,
                          time=new.time, n1=new.n1, m2=new.m2, u2=new.u2,
@@ -419,113 +542,118 @@ sampfrac <- as.vector(sampfrac)
                          mean.muTT=mean.muTT, sd.muTT=sd.muTT,
                          tauTT.alpha=tauTT.alpha,tauTT.beta=tauTT.beta,
                          debug=debug, debug2=debug2,  
-			 engine=engine,
-			 InitialSeed=InitialSeed)
+                  			 InitialSeed=InitialSeed, save.output.to.files=save.output.to.files)
    }
-
   ## Now to create the various summary tables of the results
 
+  ## Add the pooled Petersen estimators to the result
+  results$PP$using.all.data <-pp 
+    
   Nstrata.rel <- length(n1)
   Nstrata.cap <- ncol(expanded.m2) -1 ## don't forget that last column of m2 is number of fish never seen
 
-  ## A plot of the observered log(U) on the log scale, and the final mean log(U)
-  plot_logU <- function(title, time, n1, m2, u2, logitP.cov, results){
-    ##  Plot the observed and fitted logU values along with posterior limits
-    ##  n1, m2 (the expanded version), u2 are the raw data
-    ##  results is the summary table from WinBugs
+  # A plot of the observered log(U) on the log scale, and the final mean log(U)
+  plot.df   <- data.frame(time =new.time)
+  plot.df$logUi <-log( c((new.u2[1:Nstrata.rel]+1)*(new.n1+2)/(apply(expanded.m2[,1:Nstrata.cap],1,sum)+1), rep(NA, length(u2)-Nstrata.rel)))
 
-    Nstrata.rel <- length(n1)
-    Nstrata.cap <- ncol(m2)-1  ## remember that last column of m2 is number of fish never seen again
-    Uguess <- (u2[1:Nstrata.rel]+1)*(n1+2)/(apply(m2[,1:Nstrata.cap],1,sum)+1)  ## try and keep Uguess larger than observed values
-    plot(time[1:Nstrata.rel], log(Uguess), type="p",
-         main=paste(title,"\nFitted spline curve to raw U[i] with 95% credible intervals"),
-         sub='Open/closed circles - initial and final estimates',
-         ylab='log(U[i])',
-         xlab='Time Index', xlim=c(min(time),max(time)))  ## initial points on log scale.
+# extract the fitted U values
+  results.row.names <- rownames(results$summary)
+  etaU.row.index    <- grep("etaU", results.row.names)
+  etaU<- results$summary[etaU.row.index,]
+  plot.df$logU =etaU[,"mean"]
+  plot.df$lcl =etaU[,"2.5%"]
+  plot.df$ucl =etaU[,"97.5%"]
 
+# extract the spline values
+  logUne.row.index <- grep("logUne", results.row.names)
+  logUne<- results$summary[logUne.row.index,"mean"]
+  plot.df$spline <- results$summary[logUne.row.index,"mean"]
 
-    ## which rows contain the etaU[xx] ?
-    results.row.names <- rownames(results$summary)
-    etaU.row.index    <- grep("etaU", results.row.names)
-    etaU<- results$summary[etaU.row.index,]
+  fit.plot <- ggplot(data=plot.df, aes_(x=~new.time))+
+    ggtitle(title, subtitle="Fitted spline curve with 95% credible intervals for estimated log(U[i])")+
+    geom_point(aes_(y=~logUi), color="red", shape=1)+  # open circle
+    xlab("Time Index\nOpen/closed circles - initial and final estimates")+ylab("log(U[i])")+
+    geom_point(aes_(y=~logU), color="black", shape=19)+
+    geom_line (aes_(y=~logU), color="black")+
+    geom_errorbar(aes_(ymin=~lcl, ymax=~ucl), width=.1)+
+    geom_line(aes_(y=~spline),linetype="dashed")
 
-    ## plot the mean of the etaU
-    points(time, etaU[,"mean"], type="p", pch=19)  ## fitted values
-    lines(time, etaU[,"mean"])  ## add smoothed spline through points
-    ## plot the 2.5 -> 97.5 posterior values
-    segments(time, etaU[,"2.5%"], time, etaU[,"97.5%"])
-
-    ## plot the spline curve before the error term is added.
-    ## extract the bU coefficients
-    logUne.row.index <- grep("logUne", results.row.names)
-    logUne<- results$summary[logUne.row.index,"mean"]
-    points(time, logUne, type="p", pch=20)
-    lines(time, logUne, lty=2)  ## plot the curve
-  }
-
-
-  pdf(file=paste(prefix,"-logU.pdf",sep=""))
-  plot_logU(title=title, time=new.time, n1=new.n1, m2=expanded.m2, u2=new.u2, results=results)
-  dev.off()
+  if(save.output.to.files)ggsave(plot=fit.plot, filename=paste(prefix,"-fit.pdf",sep=""), height=6, width=10, units="in")
+  results$plots$fit.plot <- fit.plot
 
+  
+  ## plot the logitP over time
   logitP.plot <- plot_logitP(title=title, time=new.time, n1=new.n1, m2=expanded.m2, u2=new.u2, logitP.cov=new.logitP.cov, results=results)
-  ggsave(plot=logitP.plot, filename=paste(prefix,"-logitP.pdf",sep=""), height=6, width=10, units="in")
+  if(save.output.to.files)ggsave(plot=logitP.plot, filename=paste(prefix,"-logitP.pdf",sep=""), height=6, width=10, units="in")
   results$plots$logitP.plot <- logitP.plot
 
+  
   ## Look at autocorrelation function for Ntot
-  pdf(file=paste(prefix,"-Utot-acf.pdf",sep=""))
-  acf(results$sims.matrix[,"Utot"], main=paste(title,"\nAutocorrelation function for U total"))
-  dev.off()
+  mcmc.sample <- data.frame(parm="Utot", sample=results$sims.matrix[,"Utot"], stringsAsFactors=FALSE)
+  acf.Utot.plot <- plot_acf(mcmc.sample)
+  if(save.output.to.files)ggsave(plot=acf.Utot.plot, filename=paste(prefix,"-Utot-acf.pdf",sep=""), height=4, width=6, units="in")
+  results$plots$acf.Utot.plot <- acf.Utot.plot
 
+  
   ## Look at the shape of the posterior distribution
-  pdf(file=paste(prefix,"-Ntot-posterior.pdf",sep=""))
-  plot( x=density(as.vector(results$sims.array[,,"Ntot"])),
-       main=paste(title,'\nPosterior density plot of N-total'),
-       sub ="Vertical lines mark 2.5th and 97.5th percentile")
-  abline(v=results$summary["Ntot",c("2.5%","97.5%")])  ## add vertical reference lines
-  dev.off()
+  mcmc.sample1 <- data.frame(parm="Utot", sample=results$sims.matrix[,"Utot"], stringsAsFactors=FALSE)
+  mcmc.sample2 <- data.frame(parm="Ntot", sample=results$sims.matrix[,"Ntot"], stringsAsFactors=FALSE)
+  mcmc.sample <- rbind(mcmc.sample1, mcmc.sample2)
+  post.UNtot.plot <- plot_posterior(mcmc.sample)
+  post.UNtot.plot
+  if(save.output.to.files)ggsave(plot=post.UNtot.plot, filename=paste(prefix,"-UNtot-posterior.pdf",sep=""),
+                               height=ifelse(length(unique(mcmc.sample$parm))<=2,4,6), width=6, units="in")
+  results$plots$post.UNtot.plot <- post.UNtot.plot
 
-  pdf(file=paste(prefix,"-Utot-posterior.pdf",sep=""))
-  plot( x=density(as.vector(results$sims.array[,,"Utot"])),
-       main=paste(title,'\nPosterior density plot of U-total'),
-       sub ="Vertical lines mark 2.5th and 97.5th percentile")
-  abline(v=results$summary["Utot",c("2.5%","97.5%")])  ## add vertical reference lines
-  dev.off()
 
   #browser()
   ## Bayesian P-values
-  pdf(file=paste(prefix,"-GOF.pdf",sep=""))
+  #browser()
   discrep <-PredictivePosterior.TSPNDENP(new.n1, expanded.m2, new.u2,
                                          new.logitP.fixed,
                                          expit(results$sims.list$logitP),
                                          round(results$sims.list$U),
                                          results$sims.list$Theta,
-                                         Delta.max, engine)
+                                         Delta.max)
+  #browser()
   gof <- PredictivePosteriorPlot.TSPNDE (discrep)
-  dev.off()
+  if(save.output.to.files)ggsave(gof[[1]],filename=paste(prefix,"-GOF.pdf",sep=""),  height=8, width=8, units="in", dpi=300 )
+  results$plots$gof.plot <- gof
 
-   varnames <- names(results$sims.array[1,1,])  # extract the names of the variables
-   # First do the trace plots of logitP
-   pdf(file=paste(prefix,"-trace-logitP.pdf",sep=""))
-   parm.names <- varnames[grep("^logitP", varnames)]
-   trace_plot(title=title, results=results,
-       parms_to_plot=parm.names, panels=c(3,2))
-   dev.off()
+  # create traceplots of logU, U, and logitP (along with R value) to look for non-convergence
+  # the plot_trace will return a list of plots (one for each page as needed)
+  varnames <- names(results$sims.array[1,1,])  # extract the names of the variables 
 
-   # now for the traceplots of logU (etaU), Utot, and Ntot
-   pdf(file=paste(prefix,"-trace-logU.pdf",sep=""))
-   parm.names <- varnames[c(grep("Utot",varnames), grep("Ntot",varnames), grep("^etaU", varnames))]
-   trace_plot(title=title, results=results,
-       parms_to_plot=parm.names, panels=c(3,2))
-   dev.off()
+  
+  # Trace plots of logitP
+  trace.plot <- plot_trace(title=title, results=results, parms_to_plot=varnames[grep("^logitP", varnames)])
+  if(save.output.to.files){
+     pdf(file=paste(prefix,"-trace-logitP.pdf",sep=""))
+     l_ply(trace.plot, function(x){plot(x)})
+     dev.off()
+  }
+  results$plots$trace.logitP.plot <- trace.plot
+
+  
+  # now for the traceplots of logU (etaU), Utot, and Ntot
+  trace.plot <- plot_trace(title=title, results=results, parms_to_plot=varnames[c(grep("Utot",varnames), grep("Ntot",varnames), grep("^etaU", varnames))])
+  if(save.output.to.files){
+     pdf(file=paste(prefix,"-trace-logU.pdf",sep=""))
+     l_ply(trace.plot, function(x){plot(x)})
+     dev.off()
+  }
+  results$plots$trace.logU.plot <- trace.plot
 
 
-  sink(results.filename, append=TRUE)
+  sink(report, append=TRUE)
 
-  ## Global summary of results
+   ## Global summary of results
   cat("\n\n*** Summary of MCMC results *** \n\n")
-  print(results, digits.summary=3)
-
+  save.max.print <- getOption("max.print")
+  options(max.print=.Machine$integer.max)
+  print(results, digits.summary=3)#, max=.Machine$integer.max)
+  options(max.print=save.max.print)
+  
   cat("\n\n*** Alternate DIC computation based on p_D = var(deviance)/2 \n")
   results.row.names <- rownames(results$summary)
   deviance.row.index<- grep("deviance", results.row.names)
@@ -559,12 +687,19 @@ sampfrac <- as.vector(sampfrac)
   rownames(temp) <- c("Mean", "Sd")
   print(round(temp,2))
 
+  # Add the runtiming to the output object
+  results$runTime <- temp
+
 
   cat("\n\n")
   cat(paste("*** end of fit *** ", date()))
 
   sink()
 
+  # save the report to a files?
+  if(save.output.to.files)writeLines(stdout, results.filename)
+  results$report <- stdout
+
 
   ## add some of the raw data to the bugs object for simplicity in referencing it later
   results$data <- list( time=time, n1=n1, m2=m2, u2=u2, sampfrac=sampfrac,
@@ -572,7 +707,8 @@ sampfrac <- as.vector(sampfrac)
                        bad.n1=bad.n1, bad.m2=bad.m2, bad.u2=bad.u2,
                        logitP.cov=logitP.cov,
                        version=version, date_run=date(),title=title)
-  results$gof <- gof
 
+  results$gof <- gof
+  
   return(results)
 } ## end of function
